/******************************************************************************
 * Join_Command - Hook up a player to an invited game.
 * $Id: Join_Command.java,v 1.12 2011/06/08 20:52:38 dick Exp $
 * 
 * jIBS - A Java clone of FIBS
 * Copyright 2002 - Axel Leucht
 * Copyright 2011 - Dick Balaska - BuckoSoft, Corp.
 * 
 * $Log: Join_Command.java,v $
 * Revision 1.12  2011/06/08 20:52:38  dick
 * Start match/game cleanups.
 * player[12] becomes player[XO].
 *
 * Revision 1.11  2011/06/07 20:16:03  dick
 * Fix crawford/postcrawford handling.
 *
 * Revision 1.10  2011/06/06 03:18:26  dick
 * JibsMessages belongs in n.s.jibs.server, not n.s.jibs.gui
 *
 * Revision 1.9  2011/06/06 02:52:17  dick
 * getAllClients() comes from JibsServer now.
 * Reset player toggles to new game state.
 *
 * Revision 1.8  2011/06/05 18:26:33  dick
 * log is a method of JibsServer.  Always write screen logs to the log file.
 * So, no need to pass write-to-logs=true from everywhere.
 *
 * Revision 1.7  2011/06/05 06:01:03  dick
 * Use less parameters in game constructors.  We don't need playerX and playerXName, that is redundant.
 *
 * Revision 1.6  2011/06/03 17:04:07  dick
 * getPlayerXPoints() becomes getPlayerXScore().
 *
 * Revision 1.5  2011/06/03 04:15:49  dick
 * Attributes player[12]Got becomes player[12]Score.
 *
 * Revision 1.4  2011/06/02 19:29:16  dick
 * When a player starts a game, he is no longer a watcher.
 *
 * Revision 1.3  2011/06/01 20:12:26  dick
 * informNewGame() tells everybody about the new match, not just the players.
 *
 * Revision 1.2  2011/05/31 13:02:43  dick
 * Bucko headers.
 *
 */

/* 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * The Original Code is jIBS, <https://sourceforge.net/projects/fibs/>.
 * The Initial Developer of the Original Code is Axel Leucht.
 * 
 */
package net.sourceforge.jibs.command;

import net.sourceforge.jibs.backgammon.BackgammonBoard;
import net.sourceforge.jibs.backgammon.Die;
import net.sourceforge.jibs.backgammon.JibsGame;
import net.sourceforge.jibs.backgammon.JibsMatch;
import net.sourceforge.jibs.server.JibsMessages;
import net.sourceforge.jibs.server.JibsQuestion;
import net.sourceforge.jibs.server.JibsServer;
import net.sourceforge.jibs.server.Player;
import net.sourceforge.jibs.util.ClipConstants;
import net.sourceforge.jibs.util.InviteQuestion;
import net.sourceforge.jibs.util.JibsNewGameData;
import net.sourceforge.jibs.util.JibsWriter;
import net.sourceforge.jibs.util.ResumeQuestion;
import net.sourceforge.jibs.util.SavedGameParam;

/** Hook up a player to an invited game.
 *
 * @author Axel Leucht
 * @author Dick Balaska
 * @version $Revision: 1.12 $ <br> $Date: 2011/06/08 20:52:38 $
 * @see <a href="http://cvs.buckosoft.com/Projects/jIBS/jIBS/src/main/java/net/sourceforge/jibs/command/Join_Command.java">cvs Join_Command.java</a>
 */
public class Join_Command implements JibsCommand {
	private JibsServer jibsServer;

	/** Execute the <code>join</code> command.
	 */
	public boolean execute(JibsServer jibsServer, Player player,
			String strArgs, String[] args) {
		this.jibsServer = jibsServer;

		JibsGame game = player.getGame();
		Player opponent = null;
		int matchLength = 0;
		JibsMatch matchVersion = null;

		if (game != null) {
			BackgammonBoard board = game.getBackgammonBoard();

			if (board.isEnded()) {
				return doJoinForNewMatch(player, args);
			} else {
				if (!board.isEnded()) {
					opponent = board.getOpponent(player);
					matchLength = board.getMatchlength();
					matchVersion = board.getMatchVersion();
					board.setJoinPlayer(player);

					if ((board.getJoinPlayer1() != null) &&
							(board.getJoinPlayer2() != null)) {
						// start next game
						BackgammonBoard nextBoard = null;
						nextBoard = new BackgammonBoard(jibsServer,
								board.getPlayerX(),
								board.getPlayerO(),
								matchLength,
								matchVersion);
						nextBoard.setPlayerXScore(board.getPlayerXScore());
						nextBoard.setPlayerOScore(board.getPlayerOScore());

						boolean useCrawford = player.checkToggle("crawford") &&
						opponent.checkToggle("crawford");
						if (board.isCrawFordGame())
							nextBoard.setPostCrawFordGame(true);

						if (useCrawford && !board.isPostCrawfordGame() || matchLength == 1) {
							int diff1 = 0;
							int diff2 = 0;

							diff1 = board.getMatchlength() -
							board.getPlayerXScore();

							if (diff1 == 1) {
								nextBoard.setCrawFordGame(true);
								nextBoard.setMayDoubleX(0);
								nextBoard.setMayDoubleO(0);
							}

							diff2 = board.getMatchlength() -
							board.getPlayerOScore();

							if (diff2 == 1) {
								nextBoard.setCrawFordGame(true);
								nextBoard.setMayDoubleO(0);
								nextBoard.setMayDoubleX(0);
							}
						}

						board = nextBoard;
						game.setBackgammonBoard(board);

						greet(board.getPlayerX(), board.getPlayerO(),
								board.getMatchVersion(), board.getMatchlength());
// XXX:
						JibsNewGameData jibsNewGameData = jibsNewGameData(game.getPlayerX(),
								game.getPlayerO());
						player.getGame().getBackgammonBoard().setpDie1(jibsNewGameData.getDie1());
						player.getGame().getBackgammonBoard().setpDie2(jibsNewGameData.getDie2());

						player.getGame().getBackgammonBoard().setJoinPlayer1(null);
						player.getGame().getBackgammonBoard().setJoinPlayer2(null);
						player.startGame(game,
								
								jibsNewGameData.getTurn(),
								board.getMayDoubleX(),
								board.getMayDoubleO());
					}
				}
			}
		} else {
			return doJoinForNewMatch(player, args);
		}
		return true;
	}

	private boolean doJoinForNewMatch(Player player, String[] args) {
		JibsMessages jibsMessages = jibsServer.getJibsMessages();
		Player opponent = null;
		int matchlength = 0;
		JibsMatch matchVersion = null;
		JibsGame game = player.getGame();

		if (args.length <= 1) {
			// m_join_no_user=** Error: Join who?
			String msg = jibsMessages.convert("m_join_no_user");
			player.println(msg);
		} else {
			opponent = jibsServer.getPlayer(args[1]);

			if (opponent == null) {
				// m_join_no_invite=** %0 didn't invite you.
				Object[] obj = { args[1] };
				String msg = jibsMessages.convert("m_join_no_invite", obj);
				player.println(msg);
			} else {
				player.setOpponent(opponent);
				opponent.setOpponent(player);

				JibsQuestion question = opponent.getQuestion();

				if (question instanceof ResumeQuestion) {
					informNewGame(player, opponent);

					ResumeQuestion resumeQuestion = (ResumeQuestion) question;
					SavedGameParam resumeData = resumeQuestion.getResumeData();
					JibsGame resumeGame = new JibsGame(jibsServer, resumeData);
					BackgammonBoard board = resumeGame.getBackgammonBoard();
					resumeGame.setJibsServer(jibsServer);

					int startturn = board.getTurn();
					Die dice1 = board.getpDie1();
					Die dice2 = board.getpDie2();
					int player1Score = board.getPlayerXScore();
					int player2Score = board.getPlayerOScore();

					player.setGame(resumeGame);
					opponent.setGame(resumeGame);
					player.setTogglesForNewGame();
					opponent.setTogglesForNewGame();
					resumeGame.resumeGame(resumeData, startturn, dice1, dice2,
							player1Score, player2Score);
				}

				if (question instanceof InviteQuestion) {
					InviteQuestion inviteQuestion = (InviteQuestion) question;
					matchlength = inviteQuestion.getMatchLength();
					matchVersion = inviteQuestion.getMatchVersion();
					player.stopWatching();
					opponent.stopWatching();
					informNewGame(player, opponent);

					// m_player_joined=** Player %0 has joined you for a %1
					// point match.
					Object[] obj = new Object[] { player.getName(), matchlength };
					String msg = jibsMessages.convert("m_player_joined", obj);
					opponent.getOutputStream().println(msg);
					greet(player, opponent, matchVersion, matchlength);

					JibsNewGameData jibsNewGameData = jibsNewGameData(player,
							opponent);

					game = new JibsGame(jibsServer,
							jibsNewGameData.getPlayerX(),
							jibsNewGameData.getPlayerO(),
							matchlength, 0, 0, matchVersion);
					player.setGame(game);
					opponent.setGame(game);

					BackgammonBoard board = game.getBackgammonBoard();
					board.setpDie1(jibsNewGameData.getDie1());
					board.setpDie2(jibsNewGameData.getDie2());
					game.setBackgammonBoard(board);
					JibsGame.deleteGames(jibsServer,
							jibsNewGameData.getPlayerX(),
							jibsNewGameData.getPlayerO());
					// m_start_match=%0 and %1 start a new %2-point match.
					obj = new Object[] {
							jibsNewGameData.getPlayerX().getName(),
							jibsNewGameData.getPlayerO().getName(),
							matchlength
					};
					msg = jibsMessages.convert("m_start_match", obj);
					jibsServer.log(msg);		// TODO: this should get outted to all clients

					jibsNewGameData.getPlayerX().startGame(game, jibsNewGameData.getTurn(),
							1, 1);
				}
			}
		}
		return true;
	}

	/** Send the "Starting a new game" message to both players and watchers.
	 * @param player1 One player in the game
	 * @param player2 Another player in the game
	 * @param matchVersion  {@link JibsMatch#nPointMatch} or {@link JibsMatch.unlimitedMatch}
	 * @param matchlength The number of points in the match
	 */
	public void greet(Player player1, Player player2, JibsMatch matchVersion,
			int matchlength) {
		JibsMessages jibsMessages = jibsServer.getJibsMessages();
		String playerName = player1.getName();
		String oppplayerName = player2.getName();
		JibsWriter out1 = player1.getOutputStream();

		JibsWriter out2 = player2.getOutputStream();

		// m_start_match_with=Starting a new game with %0.
		Object[] obj = { oppplayerName };
		String msg = jibsMessages.convert("m_start_match_with", obj);
		out1.println(msg);

		if (matchVersion.getVersion() == JibsMatch.nPointMatch) {
			// m_start_match_with=Starting a new game with %0.
			obj = new Object[] { playerName };
			msg = jibsMessages.convert("m_start_match_with", obj);
			out2.println(msg);

			// m_start_match=%0 and %1 start a new %2-point match.
			oppplayerName = player2.getName();
			obj = new Object[] {
					playerName, oppplayerName, Integer.valueOf(matchlength)
			};
			msg = jibsMessages.convert("m_start_match", obj);
			player1.informPlayers(msg, player2);
			// message needs to be sent also to both watcher lists
			player1.informWatchers("m_start_match", obj, true);
			player2.informWatchers("m_start_match", obj, true);
		} else {
			// m_you_play_unlimited=** You are now playing an unlimited match
			// with %1
			obj = new Object[] { playerName };
			msg = jibsMessages.convert("m_you_play_unlimited", obj);
			out2.println(msg);
			// m_start_match_unlimited=%0 and %1 start an unlimited match.
			oppplayerName = player2.getName();
			obj = new Object[] { playerName, oppplayerName };
			msg = jibsMessages.convert("m_start_match_unlimited", obj);
			player1.informPlayers(msg, player2);
		}
	}

	private JibsNewGameData jibsNewGameData(Player playerX, Player playerO) {
		JibsNewGameData jngd = null;
		Die die1 = new Die(jibsServer);
		Die die2 = new Die(jibsServer);
		int startTurn = determineStartPlayer(playerX, playerO, die1, die2);
		jngd = new JibsNewGameData(playerX, playerO, startTurn,	die1, die2);
/*
		switch (startTurn) {
		case 1:
			jngd = new JibsNewGameData(playerO, playerX, -1,
					die2, die1);

			break;

		case -1:
			jngd = new JibsNewGameData(playerX, playerO, -1,
					die1, die2);

			break;
		}
*/
		return jngd;
	}

	/** Everybody gets notified of these two player's changed status.
	 * @param player The one player that changed.
	 * @param opponent The other player that changed.
	 */
	private void informNewGame(Player player, Player opponent) {
		final String s = ClipConstants.CLIP_WHO_INFO + " ";
		final String t = ClipConstants.CLIP_WHO_END + " ";
		for (Player p : this.jibsServer.getAllClients().values()) {
			String pre = "";
			String post = "";
			if (p.canCLIP()) {
				pre = s;
				post = t;
			}
			p.getOutputStream().print(pre);
			Player.whoinfo(p.getOutputStream(), player, player);
			p.getOutputStream().println(post);
			p.getOutputStream().print(pre);
			Player.whoinfo(p.getOutputStream(), player, opponent);
			p.getOutputStream().println(post);
		}
	}

	private int determineStartPlayer(Player playerX, Player playerO,
			Die rollDice1, Die rollDice2) {
		JibsMessages jibsMessages = jibsServer.getJibsMessages();

		String playerName = playerX.getName();
		String oppplayerName = playerO.getName();
		String msg = null;
		Object[] obj = null;

		do {
			rollDice1.roll();
			rollDice2.roll();
//			rollDice1.setValue(3); // joining Player
//			rollDice2.setValue(1); // inviting Player

			// m_both_roll=%0 rolled %1, %2 rolled %3
			obj = new Object[] {
					"You", Integer.valueOf(rollDice1.getValue()),
					oppplayerName, Integer.valueOf(rollDice2.getValue())
			};
			msg = jibsMessages.convert("m_both_roll", obj);
			playerX.println(msg);
			// m_both_roll=%0 rolled %1, %2 rolled %3
			obj = new Object[] {
					playerName, Integer.valueOf(rollDice1.getValue()),
					oppplayerName, Integer.valueOf(rollDice2.getValue())
			};
			playerX.informWatchers("m_both_roll", obj, true);
			playerO.informWatchers("m_both_roll", obj, true);
			// m_both_roll=%0 rolled %1, %2 rolled %3
			obj = new Object[] {
					"You", Integer.valueOf(rollDice2.getValue()), playerName,
					Integer.valueOf(rollDice1.getValue())
			};
			msg = jibsMessages.convert("m_both_roll", obj);
			playerO.println(msg);
		} while (rollDice1.getValue() == rollDice2.getValue());

		if (rollDice1.getValue() > rollDice2.getValue()) {
			// m_your_turn=It's your turn to move.
			msg = jibsMessages.convert("m_your_turn");
			playerX.println(msg);
			// m_makes_first_move=%0 makes the first move.
			obj = new Object[] { playerName };
			msg = jibsMessages.convert("m_makes_first_move", obj);
			playerO.println(msg);
			playerX.informWatchers("m_makes_first_move", obj, true);
			playerO.informWatchers("m_makes_first_move", obj, true);

			return -1;
		} else {
			// m_your_turn=It's your turn to move.
			msg = jibsMessages.convert("m_your_turn");
			playerO.println(msg);
			// m_makes_first_move=%0 makes the first move.
			obj = new Object[] { oppplayerName, };
			msg = jibsMessages.convert("m_makes_first_move", obj);
			playerX.println(msg);
			playerX.informWatchers("m_makes_first_move", obj, true);
			playerO.informWatchers("m_makes_first_move", obj, true);

			return 1;
		}
	}
}
