/******************************************************************************
 * Leave_Command - Tell the client about jIBS.
 * $Id: Leave_Command.java,v 1.5 2011/06/08 20:07:10 dick Exp $
 * 
 * jIBS - A Java clone of FIBS
 * Copyright 2002 - Axel Leucht
 * Copyright 2011 - Dick Balaska - BuckoSoft, Corp.
 * 
 * $Log: Leave_Command.java,v $
 * Revision 1.5  2011/06/08 20:07:10  dick
 * Break up the Leave command so that we can alternately call it with a shuttingDown parameter.
 * If jIBS is going down, then we don't want to send "Noone to leave" messages to half of the players.
 *
 * Revision 1.4  2011/06/06 03:18:26  dick
 * JibsMessages belongs in n.s.jibs.server, not n.s.jibs.gui
 *
 * Revision 1.3  2011/06/05 18:26:33  dick
 * log is a method of JibsServer.  Always write screen logs to the log file.
 * So, no need to pass write-to-logs=true from everywhere.
 *
 * Revision 1.2  2011/05/31 13:02:43  dick
 * Bucko headers.
 *
 */

/* 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * The Original Code is jIBS, <https://sourceforge.net/projects/fibs/>.
 * The Initial Developer of the Original Code is Axel Leucht.
 * 
 */
package net.sourceforge.jibs.command;

import net.sourceforge.jibs.backgammon.BackgammonBoard;
import net.sourceforge.jibs.backgammon.JibsGame;
import net.sourceforge.jibs.server.JibsMessages;
import net.sourceforge.jibs.server.JibsServer;
import net.sourceforge.jibs.server.Player;
import net.sourceforge.jibs.util.JibsWriter;

/** A Player leaves the match.
 *
 * @author Axel Leucht
 * @author Dick Balaska
 * @version $Revision: 1.5 $ <br> $Date: 2011/06/08 20:07:10 $
 * @see <a href="http://cvs.buckosoft.com/Projects/jIBS/jIBS/src/main/java/net/sourceforge/jibs/command/Leave_Command.java">cvs Leave_Command.java</a>
 */
public class Leave_Command implements JibsCommand {

	/** Execute the Leave command.
	 */
	public boolean execute(JibsServer jibsServer, Player player,
			String strArgs, String[] args) {
		execute(jibsServer, player, false);
		return(true);
	}

	/** Execute the Leave command.
	 * @param jibsServer The jibsServer
	 * @param player The player that is leaving the match
	 * @param shuttingDown true = system is shutting down, don't output messages, just save the game.
	 */
	public void execute(JibsServer jibsServer, Player player, boolean shuttingDown) {
		JibsWriter out = player.getOutputStream();
		JibsMessages jibsMessages = jibsServer.getJibsMessages();
		String msg = null;
		JibsGame game = player.getGame();

		if (game == null) {
			if (!shuttingDown) {
				msg = jibsMessages.convert("m_leave_error");
				out.println(msg);
			}
		} else {
			BackgammonBoard board = game.getBackgammonBoard();
			msg = jibsMessages.convert("m_leave");
			out.println(msg);

			Player opponent = board.getOpponent(player);
			Object[] obj = { player.getName() };

			msg = jibsMessages.convert("m_other_leave", obj);
			jibsServer.log(msg);
			opponent.println(msg);
			game.save();
		}
	}
}
