/******************************************************************************
 * Motd_Command.java - Send the Message Of The Day to the client.
 * $Id: Motd_Command.java,v 1.3 2011/05/31 13:02:43 dick Exp $
 * 
 * jIBS - A Java clone of FIBS
 * Copyright 2002 - Axel Leucht
 * Copyright 2011 - Dick Balaska - BuckoSoft, Corp.
 * 
 * $Log: Motd_Command.java,v $
 * Revision 1.3  2011/05/31 13:02:43  dick
 * Bucko headers.
 *
 * Revision 1.2  2011/05/30 04:22:25  dick
 * Read the motd from the resources.
 *
 */

/* 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * The Original Code is jIBS, <https://sourceforge.net/projects/fibs/>.
 * The Initial Developer of the Original Code is Axel Leucht.
 * 
 */
package net.sourceforge.jibs.command;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.PrintWriter;

import net.sourceforge.jibs.server.JibsServer;
import net.sourceforge.jibs.server.Player;
import net.sourceforge.jibs.util.ClipConstants;

/** Send the Message Of The Day to the client.
 *
 * @author Axel Leucht
 * @author Dick Balaska
 * @version $Revision: 1.3 $ <br> $Date: 2011/05/31 13:02:43 $
 * @see <a href="http://cvs.buckosoft.com/Projects/jIBS/jIBS/src/main/java/net/sourceforge/jibs/command/Motd_Command.java">cvs Motd_Command.java</a>
 */
public class Motd_Command implements JibsCommand {
	private void output_motd(JibsServer jibsServer, Player player) {
		BufferedReader reader = null;

		try {
			String	resource = jibsServer.getResource( "motd" );
			File           motdFile = new File(resource);
			if (motdFile.canRead()) {
				reader = new BufferedReader(new FileReader(motdFile));
			} else {
				InputStream is = ClassLoader.getSystemResourceAsStream("conf/motd.txt");
				reader = new BufferedReader(new InputStreamReader(is));
			}

			PrintWriter out = player.getOutputStream();
			String line = "";

			if (player.canCLIP()) {
				out.println(ClipConstants.CLIP_MOTD_BEGIN);
			}

			do {
				line = reader.readLine();

				if (line != null) {
					if (line.equals("")) {
						line = " ";
					}

					out.println(line);
				}
			} while (line != null);

			if (player.canCLIP()) {
				out.println(ClipConstants.CLIP_MOTD_END);
			}

			reader.close();
			out.flush();
		} catch (FileNotFoundException e) {
			jibsServer.logException(e);
		} catch (IOException e) {
			jibsServer.logException(e);
		} finally {
			try {
				if (reader != null) {
					reader.close();
				}
			} catch (IOException e) {
				jibsServer.logException(e);
			}
		}
	}

	public boolean execute(JibsServer jibsServer, Player player,
			String strArgs, String[] args) {
		output_motd(jibsServer, player);

		return true;
	}
}
