/******************************************************************************
 * ClientWorker - Client's main communication thread.
 * $Id: ClientWorker.java,v 1.9 2011/06/08 20:54:21 dick Exp $
 * 
 * jIBS - A Java clone of FIBS
 * Copyright 2002 - Axel Leucht
 * Copyright 2011 - Dick Balaska - BuckoSoft, Corp.
 * 
 * $Log: ClientWorker.java,v $
 * Revision 1.9  2011/06/08 20:54:21  dick
 * clip_ownInfo() doesn't need the jibsServer.
 *
 * Revision 1.8  2011/06/06 03:18:26  dick
 * JibsMessages belongs in n.s.jibs.server, not n.s.jibs.gui
 *
 * Revision 1.7  2011/06/06 02:41:57  dick
 * Move Player manipulation functions from Server to JibsServer.
 * Stop passing the not-so-useful-here Server all around.
 *
 * Revision 1.6  2011/06/05 18:26:33  dick
 * log is a method of JibsServer.  Always write screen logs to the log file.
 * So, no need to pass write-to-logs=true from everywhere.
 *
 * Revision 1.5  2011/06/02 19:24:02  dick
 * Access isExitCmd() as a static.
 *
 * Revision 1.4  2011/05/31 17:48:50  dick
 * Fix all warnings from the import.
 *
 * Revision 1.3  2011/05/31 14:50:49  dick
 * Bucko headers.
 *
 * Revision 1.2  2011/05/30 04:23:15  dick
 * Don't try to set a timer with a timeout of -1.
 *
 */

/* 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * The Original Code is jIBS, <https://sourceforge.net/projects/fibs/>.
 * The Initial Developer of the Original Code is Axel Leucht.
 * 
 */
package net.sourceforge.jibs.server;

import java.io.BufferedReader;
import java.net.InetAddress;
import java.net.Socket;
import java.net.SocketException;
import java.sql.SQLException;
import java.util.Date;
import java.util.HashMap;
import java.util.StringTokenizer;

import javax.swing.SwingUtilities;

import net.sourceforge.jibs.command.JibsCommand;
import net.sourceforge.jibs.command.Login_Command;
import net.sourceforge.jibs.command.Unknown_Command;
import net.sourceforge.jibs.util.JibsTimer;
import net.sourceforge.jibs.util.JibsWriter;
import net.sourceforge.jibs.util.TimeSpan;

import org.apache.log4j.Logger;

import com.ibatis.sqlmap.client.SqlMapClient;

/**
 * This class handles a client session. An instance of this class will be
 * started as a thread whenever a  client connects to the server. This then
 * becomes the <i>server</i> for the whole interaction of  the client
 * session. This class reads commands given as input from the client and
 * dispatches it to the appropriate handler to execute it.  After a client
 * has been authentificated by <b>jIBS</b> there is a Player member
 * associated with this client session.
 *
 * @author Axel Leucht
 *
 * @see Server
 * @see Player
 * 
 * @author Dick Balaska
 * @version $Revision: 1.9 $ <br> $Date: 2011/06/08 20:54:21 $
 * @see <a href="http://cvs.buckosoft.com/Projects/jIBS/jIBS/src/main/java/net/sourceforge/jibs/server/JibsConfiguration.java">cvs JibsConfiguration.java</a>
 */
public class ClientWorker implements Runnable {
	private static final Logger logger = Logger.getLogger(ClientWorker.class);
	private BufferedReader in;
	private long inActiveMillis = 0;
	private JibsTimer inactiveTimer;
	private JibsServer jibsServer;
	private Thread jibsThread;
	private String lastCmd;
	private Date now_Cmd;
	private int once;
	private JibsWriter out;
	private Player player;
	private boolean runs;
//	private Server server;
	private Socket socket;
	private Date timeStampOfLastCmd;

	// Constructor
	ClientWorker(JibsMessages jibsMessages, JibsServer jibsServer, Socket client,
			BufferedReader in, JibsWriter out) {
//		this.server = server;
		this.socket = client;
		this.in = in;
		this.out = out;
		this.jibsServer = jibsServer;
		runs = true;
		once = 0;
		inActiveMillis = Integer.parseInt(jibsServer.getResource("ActivityTimeout"));
	}

	public static Player getPlayer(JibsServer jibsServer, Player player, String name) {

		for (Player possibleInvitee : jibsServer.getAllClients().values()) {
			if (possibleInvitee.getName().equalsIgnoreCase(name))
				return possibleInvitee;
		}

		return null;
	}

	public void connectPlayer(Player player) {
		this.player = player;

		HashMap<String, Player> allClients = jibsServer.getAllClients();

		allClients.put(player.getName(), player);

		if (jibsServer.useSwing()) {
			SwingUtilities.invokeLater(new Runnable() {
				public void run() {
					jibsServer.getStatusBar().getPlayerLabel()
					.setText("" +
							jibsServer.getAllClients().size());
				}
			});
		}
	}

	public void disConnectPlayer(Player player) {
		try {
			this.player = player;
			player.setLast_logout_date(new Date());

			HashMap<String, Player> allClients = jibsServer.getAllClients();
			allClients.remove(player.getName());

			SqlMapClient sqlClient = jibsServer.getSqlMapInstance();
			player.setLast_logout_date(new Date());
			sqlClient.update("Player.updateLogout", player);

			if (jibsServer.useSwing()) {
				SwingUtilities.invokeLater(new Runnable() {
					public void run() {
						jibsServer.getStatusBar().getPlayerLabel()
						.setText("" +
								jibsServer.getAllClients().size());
					}
				});
			}
		} catch (SQLException e) {
			jibsServer.logException(e);
		}
	}

	public boolean executeCmd(String strCmd) {
		if (player != null) {
			logger.info(player.getName() + ":" + strCmd);
			if (strCmd == null)
				logger.error("strCmd == null");
		} else {
			logger.info("<Unknown player>:" + strCmd);
		}

		if (inactiveTimer == null && inActiveMillis > 0) {
			inactiveTimer = new JibsTimer(jibsServer, this, player,
					inActiveMillis);
			jibsThread = new Thread(inactiveTimer);
			jibsThread.start();
		}

		if (inactiveTimer != null) {
			jibsThread.interrupt();

			try {
				jibsThread.join();
			} catch (InterruptedException e) {
				// jibsServer.logException(e);
				jibsThread = null;
			}

			if (inActiveMillis >= 0) {
				jibsThread = new Thread(inactiveTimer);
				jibsThread.start();
			}
		}

		String strArgs = "";
		boolean retCode = true;

		if (strCmd != null) {
			now_Cmd = new Date(new java.util.Date().getTime());

			if (timeStampOfLastCmd == null) {
				timeStampOfLastCmd = now_Cmd;
			}

			TimeSpan duration = new TimeSpan(timeStampOfLastCmd.getTime(),
					now_Cmd.getTime());
			Player player = getPlayer();

			if (player != null) {
				player.setIdle(duration);
			}

			timeStampOfLastCmd = now_Cmd;

			StringTokenizer stoken = new StringTokenizer(strCmd);
			int nrOfArgs = stoken.countTokens();
			String[] totalArgs = null;

			if (nrOfArgs > 0) {
				totalArgs = new String[nrOfArgs];
			}

			stoken = new StringTokenizer(strCmd);

			int i = 0;

			while (stoken.hasMoreTokens()) {
				totalArgs[i++] = stoken.nextToken();
			}

			// split the command and the args
			int index = -1;

			if (totalArgs != null) {
				index = strCmd.indexOf(totalArgs[0]);

				if (index >= 0) {
					strArgs = strCmd.substring(index + totalArgs[0].length());
				}

				// is there a cmd to execute the request?
				JibsCommand fc = jibsServer.getCmd(totalArgs[0]);

				if (fc != null) {
					if ((lastCmd == null) || !JibsServer.isExitCmd(lastCmd)) {
						retCode = fc.execute(jibsServer, player, strArgs,
								totalArgs);
					}

					lastCmd = strCmd;
				} else {
					JibsCommand jibscmd = new Unknown_Command();

					jibscmd.execute(jibsServer, player, strArgs, totalArgs);
				}
			}
		}

		return retCode;
	}

	public BufferedReader getInputReder() {
		return in;
	}

	public Player getPlayer() {
		return player;
	}

	public JibsWriter getPrintWriter() {
		return out;
	}

	public Socket getSocket() {
		return socket;
	}

	public void run() {
		try {
			boolean bStopLogin = false;
			String retries = jibsServer.getResource("Retries");
			int MaxRetries = Integer.parseInt(retries);

			while (runs) {
				if (!bStopLogin) {
					if (!bStopLogin && (once < MaxRetries)) {
						once++;
						jibsServer.getServer().logo(out, once);

						Login_Command login = new Login_Command();
						Player realplayer = login.login(jibsServer,
								socket, out, null, in);

						if (realplayer != null) {
							realplayer.setClientWorker(this);

							if (realplayer.getName().equalsIgnoreCase("guest")) {
								bStopLogin = true;
								runs = false;
								executeCmd("bye");
							}
						}

						if ((realplayer != null) &&
								(realplayer.is_valid(realplayer.getUserName(),
										realplayer.getPassword()))) {
							// Update the DB-record
							SqlMapClient sqlClient = jibsServer.getSqlMapInstance();
							connectPlayer(realplayer);

							bStopLogin = true;

							if (realplayer.canCLIP()) {
								realplayer.getJibsSet().getSetMap()
								.put("boardstyle", "3");
							}

							realplayer.setStreams(in, out);
							realplayer.clip_welcome(jibsServer);
							realplayer.clip_ownInfo();

							realplayer.setLast_login_date(new Date());

							Socket client = getSocket();
							InetAddress iadr = client.getInetAddress();
							realplayer.setLast_login_host(iadr.getHostName());
							sqlClient.update("Player.updateLogin", realplayer);

							executeCmd("motd");
							executeCmd("who");
						} else {
							out.println("Login incorrect");

							if ((realplayer != null) && (realplayer.canCLIP())) {
								runs = false;
								bStopLogin = true;
								realplayer.getSocket().close();
							}
						}
					} else {
						// m_too_many_errors=Too many errors for "%0" trying to
						// log in:Connection closed
						Object[] obj = { "Unknown User" };

						String msg = jibsServer.getJibsMessages()
						.convert("m_too_many_errors", obj);

						jibsServer.log(msg);

						if (player == null) {
							getSocket().close();
						} else {
							executeCmd("bye");
						}

						runs = false;
					}
				} else {
					Player clientPlayer = getPlayer();

					if (!clientPlayer.canCLIP()) {
						String Prompt = "> ";

						out.print(Prompt);
						out.flush();
					}

					String strCmd = null;

					try {
						strCmd = in.readLine();
					} catch (SocketException e1) {
						boolean doLog = e1.getMessage()
						.equalsIgnoreCase("Software caused connection abort: recv failed");
						doLog |= e1.getMessage()
						.equalsIgnoreCase("socket closed");

						if (!doLog) {
							jibsServer.logException(e1);
						}
					}

					if (strCmd == null) {
						// Client has disconnected
						executeCmd("bye");
						runs = false;
					} else {
						// the command the user has entered
						try {
							if (strCmd.equalsIgnoreCase("!!")) {
								strCmd = lastCmd; // repeat the lastCmd
							}

							executeCmd(strCmd);
						} catch (Exception e) {
							jibsServer.logException(e);
						}
					}
				}
			}
		} catch (Exception e) {
			jibsServer.logException(e);
		}
	}

	public void stopWatchThread() {
		if (jibsThread == null)
			return;
		jibsThread.interrupt();

		try {
			jibsThread.join();
		} catch (InterruptedException e) {
			jibsThread = null;
		}
	}
}
